DEFAULTS = {
    "COUCH_URL": "",
    "COUCH_DB": "",
    "COUCH_USER": "",
    "COUCH_PW": "",

    "GPKG_FILE": "",
    "GPKG_LAYER": "",
    "COL_TRONCONS": "",

    "COL_LINEAR_ID": "",
    "COL_AUTHOR": "",
    "COL_DATE_DEBUT": "",
    "COL_DATE_FIN": "",
    "COL_DESIGNATION": "",
    "COL_LIBELLE": "",
    "COL_COMMENTAIRE": "",
    "COL_LIEUDIT": "",
    "COL_POSITION_ID": "",
    "COL_COTE_ID": "",
    "COL_SOURCE_ID": "",
    "COL_CATEGORIE_DESORDRE_ID": "",
    "COL_TYPE_DESORDRE_ID": "",

    "IS_VALID": False,

    "OBS_FALLBACK_OBSERVATEUR_ID": "",
    "OBS_FALLBACK_NB_DESORDRES": "",
    "OBS_FALLBACK_URGENCE": "",
    "OBS_FALLBACK_SUITE": "",

    "PHO_FALLBACK_PHOTOGRAPH_ID": "",
    "PHO_FALLBACK_OBS_DATE": False,
    "PHO_FALLBACK_DES_GEOM": False,
    "PHO_FALLBACK_ORIENTATION": "",
    "PHO_FALLBACK_COTE": "",

    "VERBOSE": False,

    "GPKG_PATH": None,  # IMPORTANT
}

